from django.shortcuts import redirect
from django.contrib import admin, messages
from django.utils.translation import ugettext_lazy as _, ugettext
try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse
from oauth_clients import models as client_models
from . import models
from . import forms
from . import tasks


class SettingsAdmin(admin.ModelAdmin):
    list_display = ('oauth_client',)

    def has_add_permission(self, request):
        if models.Settings.objects.count() != 0:
            return False
        return super(SettingsAdmin, self).has_add_permission(request)

    def has_delete_permission(self, request, obj=None):
        return False


class AccessTokenAdmin(admin.ModelAdmin):
    list_display = ('user_id', 'username', 'simplified_scope')
    fields = ('user_id', 'username', 'simplified_scope')
    readonly_fields = fields

    def simplified_scope(self, obj):
        return obj.get_scope()
    simplified_scope.short_description = _("Scope")

    def add_view(self, request, *args, **kwargs):
        info = self.model._meta.app_label, self.model._meta.model_name
        redirect_info = client_models.Client._meta.app_label, client_models.Client._meta.model_name
        view_name = 'admin:{}_{}_changelist'.format(*info)
        settings = models.Settings.objects.first()
        if settings is None or settings.oauth_client is None:
            message = ugettext("Oauth client not configured.")
            self.message_user(request, message, messages.WARNING)
            return redirect(view_name)
        redirect_url = settings.oauth_client.complete_authorization_url(request)
        final_redirection = request.build_absolute_uri(reverse(view_name))
        authorization_url = settings.oauth_client.start_authorization_url(
            request, redirect_url, final_redirection)
        return redirect(authorization_url)


class SpreadsheetAdmin(admin.ModelAdmin):
    list_display = ('access_token', 'name', 'url')
    actions = ['import_spreadsheets']
    
    def get_form(self, request, obj=None, **kwargs):
        kwargs['form'] = forms.SpreadSheetForm
        return super(SpreadsheetAdmin, self).get_form(request, obj, **kwargs)

    def get_fields(self, request, obj=None):
        if obj is None:
            return super(SpreadsheetAdmin, self).get_fields(request, obj)
        return ('access_token', 'url', 'name', 'spreadsheet_name', 'uid', 'sheet_index',
            'sheet_name', 'omit_first_row', 'last_imported_row', 'last_imported_date',
            'input_settings')

    def get_readonly_fields(self, request, obj=None):
        if obj is None:
            return super(SpreadsheetAdmin, self).get_readonly_fields(request, obj)
        return ('spreadsheet_name', 'uid', 'sheet_name', 'last_imported_row', 'last_imported_date')

    def import_spreadsheets(self, request, queryset):
        for spreadsheet in queryset:
            tasks.import_google_spreadsheet.delay(spreadsheet.pk)
        self.message_user(request, ugettext("The selected spreadsheets are being imported."),
            messages.INFO)
    import_spreadsheets.short_description = _("Import spreadsheets")

admin.site.register(models.Settings, SettingsAdmin)
admin.site.register(models.AccessToken, AccessTokenAdmin)
admin.site.register(models.Spreadsheet, SpreadsheetAdmin)



from google.auth.credentials import Credentials as BaseCredentials


class Credentials(BaseCredentials):
    def __init__(self, token_obj):
        self.token_obj = token_obj
        self.token = token_obj.access_token
        self.expiry = None
    
    def refresh(self, *args):
        self.token_obj.do_refresh_token()



import googleapiclient.errors
import pygsheets.client
from django import forms
from django.utils.translation import ugettext
from . import models
from . import credentials


class SpreadSheetForm(forms.ModelForm):
    class Meta:
        fields = ('name', 'access_token', 'url', 'sheet_index', 'omit_first_row', 'input_settings')

    def clean(self):
        cleaned_data = super(SpreadSheetForm, self).clean()
        token = cleaned_data['access_token']
        sheet_index = cleaned_data['sheet_index']
        url = cleaned_data['url']
        try:
            creds = credentials.Credentials(token)
            client = pygsheets.client.Client(creds)
            spreadsheet = client.open_by_url(url)
            sheets = spreadsheet.worksheets()
            if sheet_index >= len(sheets):
                msg = ugettext("Wrong worksheet index.")
                self.add_error('sheet_index', msg)
        except googleapiclient.errors.HttpError as e:
            msg = ugettext("Invalid spreadsheet url.")
            if e.resp and e.resp['status'] == '403':
                msg = ugettext("You have no access to this spreadsheet.")
            self.add_error('url', msg)
        except Exception:
            msg = ugettext("Invalid spreadsheet url.")
            self.add_error('url', msg)
        return cleaned_data

    def save(self, *args, **kwargs):
        creds = credentials.Credentials(self.instance.access_token)
        client = pygsheets.client.Client(creds)
        spreadsheet = client.open_by_url(self.instance.url)
        self.instance.url = spreadsheet.url
        self.instance.uid = spreadsheet.id
        self.instance.spreadsheet_name = spreadsheet.title
        sheets = spreadsheet.worksheets()
        self.instance.sheet_name = sheets[self.instance.sheet_index].title
        return super(SpreadSheetForm, self).save(*args, **kwargs)



# -*- coding: utf-8
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _
from django.db.models.signals import pre_save


class InputGoogleSheetsConfig(AppConfig):
    name = 'inputgooglesheets'
    verbose_name = _("Inputs from Google sheets")

    def ready(self):
        from oauth_clients.models import AccessToken
        pre_save.connect(self.handle_saves_access_token, sender=AccessToken,
            dispatch_uid='0f95231d-2d04-4276-8c0a-20116898e562')

    def handle_saves_access_token(self, sender, **kwargs):
        from .utils import Utils
        instance = kwargs['instance']
        if instance.pk is None:
            Utils.fetch_token_info(instance)



from __future__ import absolute_import, unicode_literals
import logging
from celery import shared_task


logger = logging.getLogger(__name__)


@shared_task
def import_google_spreadsheet(spreadsheet_pk):
    from .models import Spreadsheet
    from .utils import Utils
    try:
        spreadsheet = Spreadsheet.objects.get(pk=spreadsheet_pk)
        logger.info("Starting import spreadsheet {}".format(spreadsheet_pk))
    except Spreadsheet.DoesNotExist:
        logger.warn("Import spreadsheet {} failed. It does not exist.".format(spreadsheet_pk))
    count = Utils.import_to_input_flow(spreadsheet)
    logger.info("Finished importing spreadsheet {}: {} rows imported".format(spreadsheet_pk, count))



import sys
import string
from collections import OrderedDict
import json
import hashlib
import requests
import pygsheets.client
from django.utils.six.moves.urllib.parse import urlencode
from django.utils.timezone import now
from inputflow.models import Input
from . import models
from . import credentials

class Utils:
    @staticmethod
    def fetch_token_info(token):
        """
        update token user_id and username if the token belongs to the oauth client
        selected in the settings object.
        """
        settings = models.Settings.objects.filter(oauth_client=token.client).first()
        if settings is not None:
            url = "https://www.googleapis.com/oauth2/v1/tokeninfo?access_token={}"
            url = url.format(token.access_token)
            response = requests.get(url)
            if response.status_code == 200:
                info = response.json()
                token.user_id = info['user_id']
                token.username = info['email']

    @staticmethod
    def get_spreadsheet(spreadsheet_obj):
        creds = credentials.Credentials(spreadsheet_obj.access_token)
        client = pygsheets.client.Client(creds)
        return client.open_by_url(spreadsheet_obj.url)

    @classmethod
    def get_worksheet(cls, spreadsheet_obj):
        spreadsheet = cls.get_spreadsheet(spreadsheet_obj)
        return spreadsheet.worksheets()[spreadsheet_obj.sheet_index]

    @staticmethod
    def worksheet_row_to_dict(row):
        cols = ['COL${}'.format(letter) for letter in string.ascii_uppercase]
        encode = lambda value: value.encode('utf-8') if sys.version_info.major == 2 else value
        result = OrderedDict(((col, encode(value)) for col, value in zip(cols, row) if value != ''))
        content = json.dumps(result)
        sha1 = hashlib.sha1(content.encode('utf-8'))
        result['_content_hash'] = sha1.hexdigest()
        return result

    @classmethod
    def get_worksheet_row(cls, worksheet, index):
        row = worksheet.get_row(index)
        non_empty = [col for col in row if col != '']
        row = cls.worksheet_row_to_dict(row)
        row['row'] = index
        if len(non_empty) == 0:
            return None
        return row

    @classmethod
    def iter_worksheet_rows(cls, spreadsheet_obj, max_count=100):
        available_rows = max_count
        row_index = spreadsheet_obj.last_imported_row
        if row_index is None:
            row_index = 2 if spreadsheet_obj.omit_first_row else 1
        else:
            row_index += 1
        worksheet = cls.get_worksheet(spreadsheet_obj)
        while available_rows:
            row = cls.get_worksheet_row(worksheet, row_index)
            if row is None:
                return
            row_index += 1
            available_rows -= 1
            yield row

    @classmethod
    def import_to_input_flow(cls, spreadsheet_obj):
        if spreadsheet_obj.input_settings is None:
            return
        imported = True
        count = 0
        while imported:
            imported = False
            for row in cls.iter_worksheet_rows(spreadsheet_obj):
                imported = True
                encoded_row = urlencode(row)
                input_row = Input()
                input_row.settings = spreadsheet_obj.input_settings
                input_row.internal_source = True
                input_row.format = 'form'
                input_row.raw_content = encoded_row
                input_row.save()
                spreadsheet_obj.last_imported_row = row['row']
                spreadsheet_obj.last_imported_date = now()
                spreadsheet_obj.save()
                input_row.notify()
                count += 1
        return count
